﻿using System;
using System.Reflection;
using System.Security.Permissions;
using System.Threading;
using Tech.CodeGeneration.Compilers;

namespace Tech.CodeGeneration.Demo
{
    class Program
    {
        static void Main()
        {
            // Простейший динамический код.
            {
                var result = CodeGenerator.ExecuteCode<int>("return 12345678 * 9 + 9;");

                Console.WriteLine(result);
            }

            // Динамический код с параметрами.
            {
                var result = CodeGenerator.ExecuteCode<double>("return a * b + c;", 
                    CodeParameter.Create("a", 9876543.21), 
                    CodeParameter.Create("b", 9),
                    CodeParameter.Create("c", -0.01));
                Console.WriteLine(result);

                var result2 = CodeGenerator.ExecuteCode<double>("return a * b + c;",
                    CodeParameter.Create("a", 12345678.9),
                    CodeParameter.Create("b", 8),
                    CodeParameter.Create("c", 0.9));

                Console.WriteLine(result2);
            }
            
            // Повторное использование динамического кода.
            {
                var code = CodeGenerator.CreateCode<DateTime>(
                    "return StartDate.AddDays(Duration);",
                    new CodeParameter("StartDate", typeof(DateTime)),
                    new CodeParameter("Duration",  typeof(int)));

                var result1 = code.Execute(DateTime.Parse("2013-01-01"), 256);
                var result2 = code.Execute(DateTime.Parse("2013-10-13"), 131);

                Console.WriteLine("{0}, {1}", result1, result2);
            }

            // Повторное использование динамического кода (Visual Basic).
            {
                var code = CodeGenerator.CreateCode<DateTime>(VB.Compiler,
                    "RETURN STARTDATE.ADDDAYS(DURATION)", null, null,
                    new CodeParameter("StartDate", typeof(DateTime)),
                    new CodeParameter("Duration", typeof(int)));

                var result1 = code.Execute(DateTime.Parse("2013-01-01"), 256);
                var result2 = code.Execute(DateTime.Parse("2013-10-13"), 131);

                Console.WriteLine("{0}, {1}", result1, result2);
            }

            // Вызов динамического кода в песочнице.
            using (var sandbox = new Sandbox())
            {
                var code = CodeGenerator.CreateCode<int>(sandbox, 
                    "return (int)(DateTime.Now - StartDate).TotalDays;",
                    new CodeParameter("StartDate", typeof(DateTime)));

                var result = code.Execute(DateTime.Parse("1962-09-17"));

                Console.WriteLine(result);
            }

            // Вызов опасного динамического кода в песочнице.
            using (var sandbox = new Sandbox())
            {
                var code = CodeGenerator.CreateCode<int>(sandbox, 
                    @"System.IO.File.Delete(filePath);",
                    new CodeParameter("filePath", typeof(string)));
                try
                {
                    code.Execute(@"c:\temp\a.txt");
                }
                catch (Exception x)
                {
                    Console.WriteLine(x.Message);
                }
            }

            // Вызов опасного динамического кода в песочнице с дополнительными правами.
            const string FILE_PATH = @"c:\temp\a.txt";
            using (var sandbox = new Sandbox(
                new FileIOPermission(FileIOPermissionAccess.AllAccess, FILE_PATH)))
            {
                var code = CodeGenerator.CreateCode<int>(sandbox, 
                    @"System.IO.File.Delete(filePath);",
                    new CodeParameter("filePath", typeof(string)));

                code.Execute(FILE_PATH);
                Console.WriteLine("OK");
            }

            // Дополнительные пространства имён и дополнительные ссылки на сборки.
            {
                var code = CodeGenerator.CreateCode<string>(@"

                    var fi = HttpUtility.ParseQueryString(qstring);
                    return fi[first] + fi[second];",

                    new [] { "System.Web" },
                    new [] { "System.dll", "System.Web.dll" },

                    new CodeParameter("qstring", typeof(string)),
                    new CodeParameter("first", typeof(string)),
                    new CodeParameter("second", typeof(string)));

                var result1 = code.Execute("Name=Иван&Surname=Петров", "Name", "Surname");
                var result2 = code.Execute("Город=Москва&Улица=Таганская", "Город", "Улица");
                Console.WriteLine("{0}; {1}", result1, result2);
            }

            // Управление из песочницы объектом переданным по ссылке.
            const string controlAlgorithm = @"
                IF Climate.Temperature > 26 THEN Unit.Working = TRUE
                IF Climate.Temperature < 22 THEN Unit.Working = FALSE";

            var unit    = new AirConditioner();
            var сlimate = new Climate { Temperature = 28 };

            using (var sandbox = new Sandbox())
            {
                var controlCode = CodeGenerator.CreateCode<int>(sandbox, 
                        VB.Compiler, controlAlgorithm, null, null,
                        CodeParameter.Create("Unit", unit),
                        CodeParameter.Create("Climate", сlimate));

                while (!Console.KeyAvailable)
                {
                    Console.WriteLine("t={0}°C", сlimate.Temperature);

                    controlCode.Execute(unit, сlimate);
                    Thread.Sleep(300);
                    сlimate.Temperature += unit.Working ? -1 : 1;
                } 
            }
        }
    }

    public class AirConditioner : MarshalByRefObject
    {
        public bool Working { get; set; }
    }

    [Serializable]
    public struct Climate
    {
        public double Temperature { get; set; }
    }
}
